//*****************************************************************************
//                                CmdLinePcr.hpp                              *
//                               ----------------                             *
//  Description : This class does the command line processing.                *
//  Started     : 22/02/2005                                                  *
//  Last Update : 12/10/2009                                                  *
//  Copyright   : (C) 2005 by M.S.Waters                                      *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#ifndef CMDLINEPCR_HPP
#define CMDLINEPCR_HPP

// System includes

#include <iostream>

// wxWindows includes

#include <wx/wx.h>
#include <wx/config.h>
#include <wx/cmdline.h>
#include <wx/filename.h>

// Application includes

#include "Version.hpp"
#include "process/PrcGNetList.hpp"

// Local constant declarations

#define CLP_NGSPICE  wxT("NGSPICE")
#define CLP_GNUCAP   wxT("GNUCAP")

#define CLP_GWAVE    wxT("GWAVE")
#define CLP_GAW      wxT("GAW")

//*****************************************************************************

class CmdLinePcr : wxCmdLineParser
{
  private :

    int            m_iArgC;
    wxChar **      m_ppsArgV;

    wxString       m_osSimrType;
    wxString       m_osAnaType;
    wxString       m_osGuileProc;
    wxString       m_osViewer;
    wxString       m_osConfigFile;
    wxString       m_osNetLstFile;
    wxArrayString  m_oasSchemFiles;
    bool           m_bCleanCfgFile;

    bool  bProcOption_a( int * piArg );  // Analysis page specifier
    bool  bProcOption_c( int * piArg );  // Rebuild/clean configuration file
    bool  bProcOption_g( int * piArg );  // Specify Guile procedure for import
    bool  bProcOption_h( int * piArg );  // Display usage and exit
    bool  bProcOption_r( int * piArg );  // Specify a configuration file
    bool  bProcOption_s( int * piArg );  // Simulator engine specifier
    bool  bProcOption_v( int * piArg );  // Display app. version and exit
    bool  bProcOption_w( int * piArg );  // Specify the waveform viewer to use
    bool  bProcSchems  ( int * piArg );  // Schematic file name/s
    bool  bProcNetList ( int * piArg );  // Netlist file name

    void  Usage( void );
    void  Version( void );

    void  SetConfig( void );
    void  CleanCfgFile( void );

  public :

    CmdLinePcr( void );
   ~CmdLinePcr( );

    bool  bClear( void );
    bool  bSetCmdLine( int iArgC, wxChar ** ppsArgV );
    bool  bProcArgs( void );

    const wxString &      rosGetSimrType   ( void ) { return( m_osSimrType    ); }
    const wxString &      rosGetAnaType    ( void ) { return( m_osAnaType     ); }
    const wxString &      rosGetGuileProc  ( void ) { return( m_osGuileProc   ); }
    const wxString &      rosGetConfigFile ( void ) { return( m_osConfigFile  ); }
    const wxString &      rosGetNetLstFile ( void ) { return( m_osNetLstFile  ); }
    const wxArrayString & roasGetSchemFiles( void ) { return( m_oasSchemFiles ); }
};

//*****************************************************************************

#endif // CMDLINEPCR_HPP
