//*****************************************************************************
//                                FileTasks.hpp                               *
//                               ---------------                              *
// Description : This is a helper class for FrmMain, it handles most of the   *
//               file operations.                                             *
// Started     : 28/05/2005                                                   *
// Last Update : 19/08/2009                                                   *
// Copyright   : (C) 2005 by M.S.Waters                                       *
// Email       : M.Waters@bom.gov.au                                          *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#ifndef FILETASKS_HPP
#define FILETASKS_HPP

// System includes

#include <iostream>

// wxWindows includes

#include <wx/wx.h>
#include <wx/config.h>

// Application includes

class FrmMain;

#include "Version.hpp"
#include "TypeDefs.hpp"
#include "base/SimnBase.hpp"
#include "process/PrcGSchem.hpp"
#include "process/PrcGNetList.hpp"

//*****************************************************************************

class FileTasks
{
  private :

    FrmMain    * m_poFrmMain;     // Pointer to application main frame
    wxConfig   * m_poCfg;         // Pointer to global configuration object
    PrcGNetList  m_oPrcGNetList;  // gNetList file conversion utility

    // Object initialization functions
    void  InitGuileProc ( void );
    void  InitNetLstFile( void );
    void  InitSchemFiles( void );

    void  DlgErrNetLst  ( void );
    void  DlgErrSchems  ( void );

    bool  bDlgOpen      ( void );
    bool  bDlgImport    ( void );
    bool  bExecImport   ( void );
    bool  bDelTmpFiles  ( void );

  public :

    FileTasks( FrmMain * poFrmMain );
   ~FileTasks( );

    void  Initialize( void ); // This function calls all the initialization fns

    bool  bIsOkGNetList( void );

    bool  bSetTitle     ( void );
    bool  bSetGuileProc ( const wxString & rosPName  );
    bool  bSetNetLstFile( const wxString & rosFName  );
    bool  bSetSchemFiles( const wxString & rosFNames );
    bool  bSetLogFile   ( const wxString & rosFName  )
                          { return( m_oPrcGNetList.bSetLogFile( rosFName ) ); }

    const wxString      &  rosGetGuileProc ( void );
    const wxString      &  rosGetNetLstFile( void );
    const wxArrayString & rosaGetSchemFiles( void );

    bool  bOpen  ( void );
    bool  bImport( void );
    bool  bReload( void );
    bool  bClose ( void );
    bool  bExit  ( void );
};

//*****************************************************************************

#endif // FILETASKS_HPP
